#include <vector>
#ifdef __APPLE__
#include <GL/glew.h>
#else
#include <OpenGL/glew.h>
#endif
#include <glm/glm.hpp>
#include <assimp/scene.h>
#include <assimp/Importer.hpp>

class Model {

	struct Mesh {
		Mesh(const aiMesh *aiMesh, GLuint progId);
		GLuint vao;
		GLuint numIndices;
	};
	
	public:
		Model(const std::string &path, GLuint progId);
		void draw() const;

		class Node {
			public:
				Node(const aiNode &aiNode, GLuint progId);
				void draw(const std::vector<Mesh> &meshes, glm::mat4 parentModel) const;
				const std::vector<Node*> &getChildren() const { return children; }
				Node* findNode(const aiNode &aiNode);
				glm::mat4 model = glm::mat4(1);
				const aiNode &ai;
			private:
				const GLuint progId;

				std::vector<Node*> children;
				std::vector<unsigned int> meshIndices;
		};
		
		Node* getRoot() { return root; }
		Node* find(const std::string &name);
	
	private:
		const GLuint progId;
		
		std::vector<Mesh> meshes;
		Node *root;
		void loadModel(const std::string &path);

		Assimp::Importer importer;
};
